float4x4 World;
float4x4 View;
float4x4 Projection;
float4 LightPosition;
float4 EyePosition;
float4 AmbientColor;
float4 DiffuseColor;
float4 SpecularColor;

struct VertexShaderInput
{
    float4 Position : POSITION;
    float3 Normal : NORMAL;
};

struct VertexShaderOutput
{
    float4 Position : POSITION;	// transformed position
    float3 L : TEXCOORD0;		// light
    float3 N : TEXCOORD1;		// surface normal
    float3 V : TEXCOORD2;		// view
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output = (VertexShaderOutput) 0;
    
    float4 worldPos = mul(input.Position, World);
    float4 viewPos = mul(worldPos, View);
    output.Position = mul(viewPos, Projection);	
    
   	// transform a surface normal from model space to world space
    output.N = mul(input.Normal, World);				
    
    // Light direction 
    output.L = LightPosition;

    // View direction (Eye position - vertex position) from eye to the vertex.
    output.V = EyePosition - worldPos;
    
   return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR
{
    // normalize our vectors.
    float3 Normal = normalize(input.N);
    float3 LightDir = normalize(input.L);
    float3 ViewDir = normalize(input.V);    
    
    // calculate diffuse light (N.L)
    float Diffuse = saturate(dot(Normal, LightDir)); 
    
    // create our reflection shader
    // R = 2 * (N.L) * N - L
    float3 Reflect = normalize(2 * Diffuse * Normal - LightDir);  

    // calculate our specular light
    float Specular = pow(saturate(dot(Reflect, ViewDir)), 20); // R.V^n

    // return our final light equation
    // I = Acolor * Aintensity + Dcolor * Dintensity * N.L + Scolor * Sintensity * (R.V)n
    return AmbientColor + DiffuseColor * Diffuse + SpecularColor * Specular; 
}

technique AmbientDiffuseSpecularLightTechnique
{
    pass Pass1
    {
        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();	// need to have ps_2_0
    }
}
